@@ -17,6 +17,7 @@ import multiprocessing
 from multiprocessing.pool import ThreadPool
 from tqdm import tqdm
 from questionary import prompt
+import coloredlogs
 
 import requests
 from getpass import getpass
@@ -31,38 +32,37 @@ HTTP_USER_AGENT_ID = "safetyculture-python-sdk"
 def interactive_login():
     login_questions = [
         {
-            'type': 'input',
-            'name': 'username',
-            'message': 'Your iAuditor username (should be your email address.)'
+            "type": "input",
+            "name": "username",
+            "message": "Your iAuditor username (should be your email address.)",
         },
-        {
-            'type': 'password',
-            'name': 'password',
-            'message': 'Your iAuditor Password'
-        }
+        {"type": "password", "name": "password", "message": "Your iAuditor Password"},
     ]
     login = prompt(login_questions)
-    username = login['username']
-    password = login['password']
+    username = login["username"]
+    password = login["password"]
     generate_token_url = "https://api.safetyculture.io/auth"
-    payload = {
-        "username": username,
-        "password": password,
-        "grant_type": "password"
-    }
+    payload = {"username": username, "password": password, "grant_type": "password"}
     headers = {
-        'content-type': "application/x-www-form-urlencoded",
-        'cache-control': "no-cache",
+        "content-type": "application/x-www-form-urlencoded",
+        "cache-control": "no-cache",
     }
-    response = requests.request("POST", generate_token_url, data=payload, headers=headers)
+    response = requests.request(
+        "POST", generate_token_url, data=payload, headers=headers
+    )
     if response.status_code == requests.codes.ok:
-        print('Token successfully obtained, continuing to export.')
-        return response.json()['access_token']
+        print("Token successfully obtained, continuing to export.")
+        return response.json()["access_token"]
     else:
-        if 'error_description' in response.json():
-            print('Error: ' + str(response.json()['error_description']))
+        if "error_description" in response.json():
+            print("Error: " + str(response.json()["error_description"]))
         else:
-            print('An error occurred calling ' + generate_token_url + ': ' + str(response.json()))
+            print(
+                "An error occurred calling "
+                + generate_token_url
+                + ": "
+                + str(response.json())
+            )
         return None
 
 
@@ -97,15 +97,15 @@ def get_user_api_token(logger):
 
 class SafetyCulture:
     def __init__(
-            self, api_token, proxy_settings=None, certificate_settings=None, ssl_verify=None
+        self, api_token, proxy_settings=None, certificate_settings=None, ssl_verify=None
     ):
         self.current_dir = os.getcwd()
         self.log_dir = self.current_dir + "/log/"
         self.api_url = "https://api.safetyculture.io/"
         self.audit_url = self.api_url + "audits/"
         self.template_search_url = (
-                self.api_url
-                + "templates/search?field=template_id&field=name&field=modified_at"
+            self.api_url
+            + "templates/search?field=template_id&field=name&field=modified_at"
         )
         self.response_set_url = self.api_url + "response_sets"
         self.get_my_groups_url = self.api_url + "share/connections"
@@ -136,8 +136,9 @@ class SafetyCulture:
         else:
             self.ssl_verify = None
         self.create_directory_if_not_exists(self.log_dir)
-        self.configure_logging()
-        logger = logging.getLogger("sp_logger")
+        logger = self.configure_logging()
+        # logger = logging.getLogger(__name__)
+        self.logger = logger
         try:
             token_is_valid = re.match("^[a-f0-9]{64}$", api_token)
             if token_is_valid:
@@ -165,32 +166,44 @@ class SafetyCulture:
         length = 0
         for i in range(0, len(lst), n):
             length += 1
-            yield lst[i: i + n]
+            yield lst[i : i + n]
 
     def raise_pool(self, list_of_audits):
         # Establish the Pool
+        # logger = self.logger
+        logger = logging.getLogger(__name__)
+        logger.debug('Raising multi-processing pool')
         self.L[:] = []
         threads = 4
+        logger.debug(f'Raising pool with {threads} threads')
         pool = ThreadPool(processes=threads)
+        logger.debug(f'Pool Raised')
         list(
             tqdm(
                 pool.imap(self.process_audit, list_of_audits), total=len(list_of_audits)
             )
         )
         pool.close()
+        logger.debug(f'Pool closed')
         pool.join()
+        logger.debug(f'Pool joined')
         return list(self.L)
 
     def process_audit(self, audit_id):
         # Appends downloaded audits to a shared list (L)
+        logger = self.logger
         audit_id = audit_id["audit_id"]
         downloaded_audit = self.get_audit(audit_id)
         if downloaded_audit:
+            logger.debug(f'Successfully download {audit_id}')
             self.L.append(downloaded_audit)
+        else:
+            logger.debug(f'{audit_id} returned as None and was not downloaded.')
 
     def requests_exceptions(
-            self, url, request_to_make, headers=None, wait_time=1, data=None
+        self, url, request_to_make, headers=None, wait_time=1, data=None
     ):
+        logger = self.logger
         if headers is None:
             headers = self.custom_http_headers
         try:
@@ -219,7 +232,7 @@ class SafetyCulture:
         except requests.exceptions.RequestException as e:
             if wait_time >= 10:
                 wait_time = 60
-                print(
+                logger.warning(
                     f"There are still issues making the request. Waiting {str(wait_time)} minutes before retrying. "
                     f"(Here is the error to help with debugging: {e})"
                 )
@@ -227,12 +240,12 @@ class SafetyCulture:
                 wait_time += 1
             elif wait_time > 1:
                 wait_time = wait_time * wait_time
-            print(
+            logger.warning(
                 f"There has been an error connecting. Waiting {str(wait_time)} minutes before retrying. "
                 f"(Here is the error to help with debugging: {e})"
             )
             time.sleep(wait_time * 30)
-            print("re-running exception func")
+            logger.warning('Wait time up - trying again.')
             response = self.requests_exceptions(
                 url, request_to_make, headers=headers, wait_time=wait_time, data=data
             )
@@ -288,7 +301,7 @@ class SafetyCulture:
         :param ex:       Exception instance to log
         :param message:  Descriptive message to describe exception
         """
-        logger = logging.getLogger("sp_logger")
+        logger = logging.getLogger(__name__)
 
         if logger is not None:
             logger.critical(message)
@@ -298,10 +311,10 @@ class SafetyCulture:
         """
         Configure logging to log to std output as well as to log file
         """
-        log_level = logging.WARNING
+        log_level = logging.DEBUG
 
         log_filename = datetime.now().strftime("%Y-%m-%d") + ".log"
-        sp_logger = logging.getLogger("sp_logger")
+        sp_logger = logging.getLogger(__name__)
         sp_logger.setLevel(log_level)
         formatter = logging.Formatter("%(asctime)s : %(levelname)s : %(message)s")
 
@@ -311,10 +324,12 @@ class SafetyCulture:
         sp_logger.addHandler(fh)
 
         sh = logging.StreamHandler(sys.stdout)
-        sh.setLevel(log_level)
+        sh.setLevel(logging.FATAL)
         sh.setFormatter(formatter)
         sp_logger.addHandler(sh)
 
+        return sp_logger
+
     def create_directory_if_not_exists(self, path):
         """
         Creates 'path' if it does not exist
@@ -335,15 +350,15 @@ class SafetyCulture:
                 raise
 
     def discover_audits(
-            self,
-            template_id=None,
-            modified_after=None,
-            modified_before=None,
-            completed=True,
-            archived=False,
-            limit=1000,
-            order="asc",
-            backlog=[],
+        self,
+        template_id=None,
+        modified_after=None,
+        modified_before=None,
+        completed=True,
+        archived=False,
+        limit=1000,
+        order="asc",
+        backlog=[],
     ):
         """
         Return IDs of all completed audits if no parameters are passed, otherwise restrict search
@@ -357,7 +372,8 @@ class SafetyCulture:
         :return:                JSON object containing IDs of all audits returned by API
         """
 
-        logger = logging.getLogger("sp_logger")
+        logger = logging.getLogger(__name__)
+        # logger = self.logger
         today = str(datetime.now().replace(microsecond=0).isoformat())
         last_modified = (
             modified_after if modified_after is not None else "2000-01-01T00:00:00.000Z"
@@ -388,21 +404,23 @@ class SafetyCulture:
             if archived == "both":
                 archived = archived
             else:
-                logger.warning("Archived must be either true or false. Defaulting to false")
+                logger.warning(
+                    "Archived must be either true or false. Defaulting to false"
+                )
         search_url = (
-                self.audit_url
-                + f"search?field=audit_id&field=modified_at&order={order}&limit={limit}"
-                  f"&modified_after={last_modified}&modified_before={modified_before}"
+            self.audit_url
+            + f"search?field=audit_id&field=modified_at&order={order}&limit={limit}"
+            f"&modified_after={last_modified}&modified_before={modified_before}"
         )
         log_string = "\nInitiating audit_discovery with the parameters: " + "\n"
         log_string += "template_id    = " + str(template_id) + "\n"
         log_string += "modified_after = " + str(last_modified) + "\n"
         log_string += "completed      = " + str(completed) + "\n"
-        logger.info(log_string)
+        logger.debug(log_string)
 
         if template_id is not None:
             log_message = "Searching against {} templates.".format(len(template_id))
-            logger.info(log_message)
+            logger.debug(log_message)
             if len(template_id) > 1:
                 for specific_id in template_id:
                     search_url += "&template=" + specific_id
@@ -426,23 +444,25 @@ class SafetyCulture:
         result = response.json() if response.status_code == requests.codes.ok else None
         number_discovered = str(result["total"]) if result is not None else "0"
         log_message = (
-                "on audit_discovery: "
-                + number_discovered
-                + " discovered using "
-                + search_url
+            "on audit_discovery: "
+            + number_discovered
+            + " discovered using "
+            + search_url
         )
 
         self.log_http_status(response.status_code, log_message)
 
         if result is not None:
             if "total" in result:
-                if int(result["total"]) > 1000 or backlog != []:
+                if int(number_discovered) > 1000 or backlog != []:
                     backlog.extend(result["audits"])
-                    # print('Backlog Length ', len(backlog))
                     if result["audits"] and len(backlog) < 500000:
-                        new_modified_after = result["audits"][len(result["audits"]) - 1][
-                            "modified_at"
-                        ]
+                        logger.info(
+                            f"More than 1000 inspections found, getting next batch. Downloaded so far: {len(backlog)}."
+                        )
+                        new_modified_after = result["audits"][
+                            len(result["audits"]) - 1
+                        ]["modified_at"]
                         self.discover_audits(
                             template_id=template_id,
                             completed=completed,
@@ -455,17 +475,17 @@ class SafetyCulture:
                         "total": len(backlog),
                         "audits": backlog,
                     }
-            if len(result['audits']) > 1000:
+            if len(result["audits"]) > 1000:
                 dupe_removal = []
                 new_audits = []
                 show_dupes = []
-                for audit in result['audits']:
-                    if audit['audit_id'] not in dupe_removal:
-                        dupe_removal.append(audit['audit_id'])
+                for audit in result["audits"]:
+                    if audit["audit_id"] not in dupe_removal:
+                        dupe_removal.append(audit["audit_id"])
                         new_audits.append(audit)
                     else:
                         show_dupes.append(audit)
-                result['audits'] = new_audits
+                result["audits"] = new_audits
         return result
 
     def discover_templates(self, modified_after=None, modified_before=None):
@@ -503,7 +523,7 @@ class SafetyCulture:
         return result
 
     def get_export_job_id(
-            self, audit_id, preference_id=None, export_format=DEFAULT_EXPORT_FORMAT
+        self, audit_id, preference_id=None, export_format=DEFAULT_EXPORT_FORMAT
     ):
         """
         Request export job ID from API and return it
@@ -556,21 +576,21 @@ class SafetyCulture:
             export_attempts = 1
             poll_status = self.authenticated_request_get(poll_url)
             status = poll_status.json()
-            logger = logging.getLogger("sp_logger")
+            logger = logging.getLogger(__name__)
             if "status" in status.keys():
                 if status["status"] == "IN_PROGRESS":
-                    logger.info(str(status["status"]) + " : " + audit_id)
+                    logger.debug(str(status["status"]) + " : " + audit_id)
                     time.sleep(delay_in_seconds)
                     return self.poll_for_export(audit_id, export_job_id)
 
                 elif status["status"] == "SUCCESS":
-                    logger.info(str(status["status"]) + " : " + audit_id)
+                    logger.debug(str(status["status"]) + " : " + audit_id)
                     return status["url"]
 
                 else:
                     if export_attempts < 2:
                         export_attempts += 1
-                        logger.info(
+                        logger.debug(
                             "attempt # {0} exporting report for: "
                             + audit_id.format(str(export_attempts))
                         )
@@ -619,7 +639,7 @@ class SafetyCulture:
             )
 
     def get_export(
-            self, audit_id, preference_id=None, export_format=DEFAULT_EXPORT_FORMAT
+        self, audit_id, preference_id=None, export_format=DEFAULT_EXPORT_FORMAT
     ):
         """
         Obtain exported document from API and return string representation of it
@@ -678,7 +698,7 @@ class SafetyCulture:
         :param page_length:     How many actions to fetch for each page of action results
         :return:                Array of action objects
         """
-        logger = logging.getLogger("sp_logger")
+        logger = logging.getLogger(__name__)
         actions_url = self.api_url + "actions/search"
         response = self.authenticated_request_post(
             actions_url,
@@ -708,7 +728,7 @@ class SafetyCulture:
         )
 
     def get_page_of_actions(
-            self, logger, date_modified, previous_page, offset=0, page_length=100
+        self, logger, date_modified, previous_page, offset=0, page_length=100
     ):
         """
         Returns a page of action search results
@@ -721,7 +741,7 @@ class SafetyCulture:
         :return: Array of action objects
         """
         if previous_page["count"] + previous_page["offset"] < previous_page["total"]:
-            logger.info(
+            logger.debug(
                 "Paging Actions. Offset: "
                 + str(offset + page_length)
                 + ". Total: "
@@ -931,15 +951,15 @@ class SafetyCulture:
         :param status_code:  http status code to log
         :param message:      to describe where the status code was obtained
         """
-        logger = logging.getLogger("sp_logger")
+        logger = logging.getLogger(__name__)
         status_description = requests.status_codes._codes[status_code][0]
         log_string = (
-                str(status_code)
-                + " ["
-                + status_description
-                + "] status received "
-                + message
+            str(status_code)
+            + " ["
+            + status_description
+            + "] status received "
+            + message
         )
-        logger.info(log_string) if status_code == requests.codes.ok else logger.error(
+        logger.debug(log_string) if status_code == requests.codes.ok else logger.error(
             log_string
         )
